;*******************************************************************************
;*                                NKC CAM by sEn                               *
;*******************************************************************************
;*                                                                             *
;*   Die Aufnahme erfolgt ber das Kamera-Modul OV7670 mit FIFO von            *
;*   OmniVision                                                                *
;*                                                                             *
;*   - Einstellung ber den I2C-Bus mit I/O-Adressen $ffffff52 - $ffffff53     *
;*     (I2C-Prozessor auf NKC Tunerkarte)                                      *
;*   - Steuerung des FIFO-Bildspeichers mit I/O - Adressen $ffffff58 / 59      *
;*                                                                             *
;*******************************************************************************
;* Versionen                                                                   *
;*******************************************************************************
;*                                                                             *
;* 1.00         26.10.2020              1. Version                             *
;*                                                                             *
;*******************************************************************************


;Ablage im Speicher, bei Assemblieren mit JADOS nicht verwenden!

;ORG $50000

;* Bibliothek

;dc.l $55AA0180
;dc.b 'CAM'
;dc.l start
;dc.l 1
;dc.b 0,0,0,0,0,0,0,0


* * * *  Programmstart  * * * *

start:
bra cam



;*** Schicht 0 - Vereinbarungsteil Konstanten


;* Konstanten

;* NKC - Basis

; CPU

CPU             equ 1                   ; 68008

; GDP

GDP1            equ $ffffff71*CPU
GDP2            equ $ffffff72*CPU       ; Control Register 2
GDP3            equ $ffffff73*CPU       ; Vergrerungsfaktor

; COL256

colcrt     equ $ffffffac*cpu
colcrtd    equ $ffffffad*cpu
colcrtb    equ $ffffffae*cpu

; Parameter COL256

coltab:          dc.b 111, 64, 80, 7, 79, 0, 64, 70, 0, 3, 0, 0, 0, 0, 0

; Key

KEYD            equ $ffffff68*CPU
KEYS            equ $ffffff69*CPU

;* I2C-Prozessor PCF8584 - auf der NKC Tuner Karte:

CTRLSTAT        equ $ffffff53*CPU ;Control-/Statusregister
REG             equ $ffffff52*CPU ;Datenregister

;* I2C-Adressen des Kameramoduls

IMAGEW          equ $42
IMAGER          equ $43

;* I2C-Subadressen / Register des Kameramoduls - Parameter gegenber Default
;* geaendert

BLUE            equ $01
RED             equ $02
VREF            equ $03
PID             equ $0A
VER             equ $0B
COM03           equ $0C
COM05           equ $0E
COM06           equ $0F
AECH            equ $10
CLKRC           equ $11
COM07           equ $12
COM08           equ $13
COM09           equ $14
RSVD01          equ $16
HSTAR           equ $17
HSTOP           equ $18
VSTAR           equ $19
VSTOP           equ $1A
MVFP            equ $1E
ADCCTR1         equ $21
ADCCTR2         equ $22
ADCCTR3         equ $23
AEW             equ $24
AEB             equ $25
VPT             equ $26
RSVD02          equ $29
HREF            equ $32
CHLF            equ $33
ARBLM           equ $34
RSVD03          equ $35
ADC             equ $37
ACOM            equ $38
OFON            equ $39
TSLB            equ $3A
COM11           equ $3B
COM12           equ $3C
COM13           equ $3D
COM14           equ $3E
COM15           equ $40
COM16           equ $41
AWBC3           equ $45
AWBC4           equ $46
AWBC5           equ $47
AWBC6           equ $48
DMPOS           equ $4D
RSVD04          equ $4E
MTX1            equ $4F
MTX2            equ $50
MTX3            equ $51
MTX4            equ $52
MTX5            equ $53
MTX6            equ $54
BRIGHT          equ $55
CONTRA          equ $56
MTXS            equ $58
AWBC7           equ $59
AWBC8           equ $5A
AWBC9           equ $5B
AWBC10          equ $5C
AWBC11          equ $5D
AWBC12          equ $5E
GFIX            equ $69
GGAIN           equ $6A
DBLV            equ $6B
AWBCTR3         equ $6C
AWBCTR2         equ $6D
AWBCTR1         equ $6E
AWBCTR0         equ $6F
SCALX           equ $70
SCALY           equ $71
SCALDCW         equ $72
SCALCLK         equ $73
REG74           equ $74
REG75           equ $75
REG76           equ $76
RSVD05          equ $78
RGB444          equ $8C
RSVD06          equ $8D
RSVD07          equ $8E
RSVD08          equ $8F
RSVD09          equ $90
RSVD10          equ $91
DMLNL           equ $92
RSVD11          equ $96
RSVD12          equ $97
RSVD13          equ $98
RSVD14          equ $99
RSVD15          equ $9A
RSVD16          equ $9B
RSVD17          equ $9C
BD50ST          equ $9D
BD60ST          equ $9E
HAECC1          equ $9F
HAECC2          equ $A0
RSVD18          equ $A1
CLKDEL          equ $A2
NTCTRL          equ $A4
BD50MAX         equ $A5
HAECC3          equ $A6
HAECC4          equ $A7
HAECC5          equ $A8
HAECC6          equ $A9
HAECC7          equ $AA
BD60MAX         equ $AB
RSVD19          equ $B0
ABLC1           equ $B1
RSVD20          equ $B2
THLST           equ $B3
RSVD21          equ $B8
SATCTR          equ $C9

;* Einstellung des Kameramoduls - nderungen der Default-Werte

;* Rahmeneinstellungen

;AGC, AWB, AEC, sharpness, denoise, colour matrix basic
pcom08          equ $E7
paew            equ $95
paeb            equ $33
pvpt            equ $E3
pcom16          equ $38
preg75          equ $03
preg76          equ $61

;automatic white balance - AWB
pblue           equ $40
pred            equ $40
pawbctr3        equ $0A
pawbctr2        equ $55
pawbctr1        equ $11
pawbctr0        equ $9f

;automatic exposure control - AEC
paech           equ $00

;automatic gain control - AGC
pcom09          equ $38
pgfix           equ $55
pggain          equ $40
preg74          equ $19
phaecc1         equ $78
phaecc2         equ $68
phaecc3         equ $D8
phaecc4         equ $D8
phaecc5         equ $F0
phaecc6         equ $90
phaecc7         equ $94

;automatic blach level calibration - ABLC
pablc1          equ $0C
pthlst          equ $82

;general settings
pcom05          equ $61
pcom06          equ $4B
pchlf           equ $03
parblm          equ $11
pcom11          equ $0A
pcom12          equ $78
pcom13          equ $08
pawbc3          equ $34
pawbc4          equ $58
pawbc5          equ 28
pawbc6          equ $3a
pdmpos          equ $40
pawbc7          equ $88
pawbc8          equ $88
pawbc9          equ $44
pawbc10         equ $67
pawbc11         equ $49
pawbc12         equ $0E
pdmlnl          equ $66
pbd50st         equ $4C
pbd60st         equ $3F
pntctrl         equ $89
pbd50max        equ $07
pbd60max        equ $08

;reserved registers
prsvd01         equ $02
padcctr1        equ $02
padcctr2        equ $91
padcctr3        equ $80
prsvd02         equ $07
prsvd03         equ $0B
padc            equ $1c
pacom           equ $71
pofon           equ $2A
prsvd04         equ $20
prsvd05         equ $04
prsvd06         equ $4f
prsvd07         equ $00
prsvd08         equ $00
prsvd09         equ $00
prsvd10         equ $00
prsvd11         equ $00
prsvd12         equ $30
prsvd13         equ $20
prsvd14         equ $20
prsvd15         equ $84
prsvd16         equ $29
prsvd17         equ $03
prsvd18         equ $0B
prsvd19         equ $8C
prsvd20         equ $0E
prsvd21         equ $0A

;mirror / flipped image
pmvfp           equ $17

;* colour matrix
pmtx1           equ $80         ;$B3
pmtx2           equ $80         ;$B3
pmtx3           equ $00
pmtx4           equ $22         ;$3D
pmtx5           equ $5E         ;$A7
pmtx6           equ $80         ;$E4
pmtxs           equ $9E

;* colour saturation
psatctr         equ $60

;* Ausgabeformat VGA mit RGB444
pcom07v         equ %00000100   ; RGB - Video
pcom07t         equ %00000110   ; RGB - Testbild
ptslb           equ %00000100   ; normal UV output
pcom15          equ %11110000   ; RGB555
prgb444         equ %00000010   ; RGB444

;* Skalierung
pcom03          equ %00001100   ; downsampling und scaling
pcom14          equ %00011001   ; pixel clock /2
pscalclk        equ %11110001   ; pixel clock /2
pclkdel         equ %00000000   ; pixel clock delay = 0
pscaldcw        equ %00010001   ; horizontal und vertikal downsampling by 2
pscalx          equ %00000000   ; Video, no horizontal scaling
pscalyt         equ %10000000   ; Testbild no verticalcal scaling
pscalyv         equ %00000000   ; Video no vertical scaling

;* Externer Takt = 12 MHz, interner Takt = 12 X 4 / 2 = 24 MHz
pdblv           equ %01000000   ; in DBLV   - externer Takt X 4
pclkrc          equ %00000001   ; in CLKRC  - externer Takt / 2

;* Erzeugung eines Bildes mit 256 X256 Bildpunkten fr die COL256:
;* - Erzeugung eines Bildes mit QVGA-Auflsung 320 X 240 Bildpunkten.
;* - Erzeugung eines pssenden Bildfensters mit 256 X 240 Bildpunkten.

phstar          equ %00011011   ; Zeilenanfang (Wert unlogisch)
phstop          equ %01011011   ;Zeilenende (Wert unlogisch)
phref           equ %10000000   ;untere 6 Bits Zeilenanfang / Zeilenende
pvstar          equ %00000010   ;Frame Start = Zeile 10
pvstop          equ %01111010   ;Frame Ende = Zeile 490
pvref           equ %00001010   ;unter 2 Bits Frame Start / Ende

;* FIFO-RAM auf Kameramodul - Steuerung Bildspeicherung

VERTsync        equ     $ffffff59*CPU   ;Abfrage Vertikal-Synchron-Signal
FIFOwr          equ     $ffffff59*CPU   ;RD Pointer Reset = Bit 0, WE = Bit 1,
                                        ;WR-Pointer Reset = Bit 2, Read Clock =
                                        ;Bit 3, Read Enable von FIFO
FIFOrd          equ     $ffffff58*CPU   ;Bild aus FIFO lesen.
                                        ;Das Lesen erzeugt den Read Clock, der
                                        ;u. a. fr den Reset bentigt wird!

;* Ablage des Bildes aus dem FIFO-RAM des Kameramoduls im NKC
;* ... im Speicherfenster der COL256

imgbegin        equ     $90000

;* Suche von JADOS in der Bibliothek
bibstart        equ     $80000
bibend          equ     $90000

;* Fehlermeldungen und Statusmeldungen

err01:          dc.b 'No CAM present!',0
err02:          dc.b 'JADOS not loaded!',0
err03:          dc.b 'No filename!',0
err04:          dc.b 'File not saved!',0
err05:          dc.b 'File not found!',0
sta01:          dc.b 's a v i n g',0
sta02:          dc.b 'l o a d i n g',0
sta03:          dc.b '     E N D  !',0

;* Texte / Figuren fr Bildschirmmaske

titel:          dc.b '***     NKC - CAM / Version 1.0     ***',0
hells:          dc.b 'Brightness (+/-)',0
konts:          dc.b 'cOntrast (+/-)',0
drives:         dc.b 'drive',0
filesavs:       dc.b 'file-save',0
fileloas:       dc.b 'file-load',0
datuhr:         dc.b 'D a t e / Ti m e',0
lblstat:        dc.b 'S t a t u s / M o d e',0
lbldir:         dc.b 'directory',0
lbljados:       dc.b 'J a d o s',0
lblfunkt:       dc.b 'F u n c t i o n',0
funkt1:         dc.b '   Scan    Clear    Mode    saVe    Load    Dir',0
funkt2:         dc.b ' dRive    esc = end',0
modvid:         dc.b 'video',0
modtst:         dc.b 'test ',0
hddrive:        dc.b '01234abcdefghijklmnopqrstuvwxyz',0
eradrv:         dc.b '  ',0
muster:         dc.b '*.*',0
fig1:           dc.b %11100111          ;Figur fuer Wertanzeige
                dc.b %11100111          ;Helligkeit und Kontrast
                dc.b %11100111
                dc.b %11100111
                dc.b %11100111
                ds 0
fcbbuff:        ds.b 48


; *** Schicht 1 - Routinen Variableninitialisierung, CI-intern, Fehler- /
;                 Statusausgabe, I2C, COL256, Uhrabfrage, JADOS-Routinen


;* Initialisierung aller Variablen

varinit:
 lea varstart(pc), A0
 movea.l A0, A4
 move varanz(A0), d0
 subq #1, d0
varclr:                         ;Aalle Bytes aller Variablen auf $00
 clr.b (A0)+
 dbra d0, varclr

;* Initialisierung Flags auf andere Startwerte

 move.b #-1, endeflag(A4)        ;bei PRG-Ende bentigt
 move.b #-1, datmin(A4)          ;damit selbe Uhrzeit nicht mehrfach angezeigt

;* Initialwerte fr Kameramodul und Stellart fr PRG

 move.b #$00, stellart(A4)      ;Stellart auf Helligkeit setzen
 move.b #$00, modus(A4)         ;Modus auf 'video' (nicht Testbild)
 move.b #3, letzteh(A4)         ;Helligkeit auf Mitte, Werte von 1-5
 move.b #3, letztek(A4)         ;Kontrast auf Mitte, Werte von 1-5
rts

;* Tastaturabfrage - intern

ciint:
 move.b KEYD.w, d0
 bmi.s ciint
 tst.b KEYS.w
rts

;* Abfrage Uhrenbaustein

uhr:
 moveq #-1, d0
 lea wertuhr(A4), A0
 move #!uhrprint, d7
 trap #1
 bcs.s uhrend
 subq #3, A0
 move.b #$00, (A0)      ; Sekunden werden im Buffer abgeschnitten

 subq #2, A0            ; Minuten im Buffer
 moveq #!wert, d7       ; Minutenwert in d0.l
 trap #1
 move.b datmin(A4), d1  ; Alter Minutenwert
 cmp.b d0, d1           ; datmin speichert den letzten Minutenwert
 beq.s uhrend           ; Wenn Minutenwert nicht gendert, keine Ausgabe!
 move.b d0, datmin(A4)  ; Neuen Minutenwert merken
 flicker:
 moveq #!sync, d7
 trap #1
 beq.s flicker
 lea wertuhr(A4), A0
 move #$11, d0
 move #312, d1
 move #203, d2
 moveq #!write, d7
 trap #1
 uhrend:
 rts

;* Ausgabe von Fehlern und Status

status:
 bsr erastat
 move.b #$11, d0
 move #30, d1
 move # 202, d2
 movea.l A1, A0
 moveq #!write, d7
 trap #1
 rts

;* Initialisierung COL256

colinit:
 lea coltab(pc), a0
 clr d1
 move #15, d3
colilp01:
 move.b d1, colcrt.w
 move.b (a0)+, colcrtd.w
 addq #1, d1
 dbra d3, colilp01
 move.b #$80, colcrtb.w
 rts

;* JADOS-Routinen

;* Pruefung, ob JADOS geladen ist

jpresent:
 move.b #-1, jados(A4)
 move.l #$55AA0180, d0          ;Bibliothekskennung
 movea.l #bibstart, A0                 ;Anfang der Suche im Speicher
 move.l #'JADO', d1             ;Name des gesuchten Eintrags
 search:
 move.l A0, d3
 cmp.l #bibend, d3
 bgt.s jperror
 move.l (A0)+, d3
 cmp.l d0, d3                   ;Bibliothekseintrag gefunden?
 bne.s search
 move.l (A0)+, d3
 cmp.l d1, d3                   ;Ersten Namensteil 'JADO' gefunden?
 bne.s search
 move.b #$00, jados(A4)
 bra jpend
 jperror:
 lea err02, A0
 move.b #$22, d0
 move #150, d1
 move #130, d2
 moveq #!write, d7
 trap #1
 jpend:
rts

;* Aktuelles Laufwerk bstimmen

actdrive:
 cmp.b #$00, jados(A4)          ;Pruefung, ob JADOS geladen ist
 bne.s actdrend
 lea eradrv(PC), A0             ;Anzeige loeschen
 move.b #$21, d0
 move #86, d1
 move #76, d2
 moveq #!write, d7
 trap #1
 moveq #53, d7                  ;JADOS GETDRIVE
 trap #6
 lea hddrive(pc), A1
 lea parbuff, A0
 ascdrive:
  move.b (A1)+, (A0)            ;Zahl oder Bustabe als ASCII
 dbra d0, ascdrive
 addq #1, A0
 move.b #':', (A0)+
 move.b #$00, (A0)              ;Pufferabschluss
 move #84,d1                    ;Rahmen neu malen wegen READ
 move #150,d2
 move #26, d3
 move #25, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 lea parbuff, A0                ;Ausgabe Laufwerl mit ':'
 move.b #$21, d0
 move #86, d1
 move #77, d2
 moveq #!write, d7
 trap #1
 actdrend:
rts

;* Neues Laufwerk setzen

setdrive:
 cmp.b #$00, jados(A4)          ;Pruefung, ob JADOS geladen ist
 bne.s setdrend
 lea parbuff(A4), A0
 move.b #$21, d0
 move #86, d1
 move #77, d2
 move #1, d3
 moveq #!read, d7               ;Eingabe neues Laufwerk - z.B. mit 'a:'
 trap #1
 lea parbuff(A4), A0
 move.b (A0), d0
 lea hddrive(pc), A1
 clr d2
 numdrive:
  addq #1, d2
  cmp.b #$00, (A1)              ;Ende Puffer fuer LW-Bezeichnungen erreicht?
  beq.s setdrend                ;Wenn ja, dann keine gltige Laufwerksangabe
  move.b (A1)+, d1
  cmp.b d0, d1                  ;Nummer fr LW-Eingabe bestimmen
 bne.s numdrive                 ;noch keine Uebereinstimmung gefunden
 subq #1, d2                    ;LW-Nummmer in JADOS beginnt beo '0'
 move d2, d0
 moveq #54, d7                  ;JADSOS - SETDRIVE
 trap #6
 setdrend:
rts

;* Directory des aktuellen Laufwerks zeigen

direc:
 lea parbuff(A4), A0
 lea muster(PC), A1             ;Wir suchen nach '*.*'
 move #630, d1                 ;Maximale Zeichenanzahl
 move #%00000000, d2            ;Keine Anzeige von Laenge, Datum, Attribut
 moveq #5, d3                   ;5 Spalten
 moveq #74, d7                  ;JADOS - DIRECTORY
 trap #6
 lea parbuff(A4), A0
 move.b #$11, d0
 move #20, d1
 move #177, d2
 moveq #!writelf, d7
 trap #1
rts

;* Bild in Datei speichern

imgsave:
 cmp.b #$00, jados(A4)          ;Pruefung, ob JADOS geladen ist
 bne savend
 lea parbuff(A4), A0
 move.b #$11, d0
 move #220, d1
 move #77, d2
 move #12, d3
 moveq #!read, d7               ;Eingabe neuer Dateiname
 trap #1
 lea parbuff(A4), A0                ;In Grossbuchstaben wandeln
 moveq #11, d7
 trap #6
 move #218,d1                   ;Rahmen neu schreiben wegen READ
 move #150,d2
 move #75, d3
 move #25, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 lea parbuff(A4), A0
 lea fcbbuff(PC), A1
 moveq #18, d7                  ;JADOS - Dateisteuerblock anlegen
 trap #6
 cmp.b #0, d0                   ;Wenn nicht '0', dann Fehler
 bne.s fcbserr
 moveq #63, d1                  ;1 Bilschirmseite der COL256 hat 64 KByte!
 lea imgbegin, A0               ;Wir speichern ab 1. Speicherstelle der COL.
 moveq #45, d7                  ;JADSOS - FILESAVE
 trap #6
 cmp.b #0, d0                   ;Wenn nicht '0', dann Fehler
 beq.s savend
 lea err04, A1
 bsr status
 move.l #10, d0
 moveq #!delay, d7
 trap #1
 bra.s savend
 fcbserr:
 lea err03, A1
 bsr status
 move.l #10, d0
 moveq #!delay, d7
 trap #1
 savend:
rts

;* Bild aus Datei laden

imgload:
 cmp.b #$00, jados(A4)          ;Pruefung, ob JADOS geladen ist
 bne loadend
 lea parbuff(A4), A0
 move.b #$11, d0
 move #400, d1
 move #77, d2
 move #12, d3
 moveq #!read, d7               ;Eingabe neuer Dateiname
 trap #1
 lea parbuff(A4), A0                ;In Grossbuchstaben wandeln
 moveq #11, d7
 trap #6
 move #398,d1                   ;Rahmen neu schreiben wegen READ
 move #150,d2
 move #75, d3
 move #25, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 lea parbuff(A4), A0
 lea fcbbuff(PC), A1
 moveq #18, d7                  ;JADOS - Dateisteuerblock anlegen
 trap #6
 cmp.b #0, d0                   ;Wenn nicht '0', dann Fehler
 bne.s fcblerr
 lea imgbegin, A0               ;Wir laden ab 1. Speicherstelle der COL
 moveq #44, d7                  ;JADSOS - FILELOAD
 trap #6
 cmp.b #0, d0                   ;Wenn nicht '0', dann Fehler
 beq.s loadend
 lea err05, A1
 bsr status
 move.l #10, d0
 moveq #!delay, d7
 trap #1
 bra.s loadend
 fcblerr:
 lea err03, A1
 bsr status
 move.l #10, d0
 moveq #!delay, d7
 trap #1
 loadend:
rts


;* I2C-Basisroutinen

;* Ansteuerung des I2C-Prozessors PCF 8584

sendctrl:                       ;Controlregister schreiben
 move.b d1, CTRLSTAT.w
 rts

senddata:                       ;Datenregister schreiben
 move.b d2, REG.w
 rts

readctrl:                       ;Statusregister lesen
 move.b CTRLSTAT.w, d1
 rts

readdata:                       ;Datenregister lesen
 move.b REG.w, d2
 rts

;* Routinen zum Initialisieren des I2C-Prozessors, zum Lesen und zum Schreiben
;* von Bytes vom und auf den I2C-Bus.

;* INITI2C initialisiert den I2C-Prozessor PCF 8584

initi2c:
 move.b #$80, d1 ;Register S0' wird ausgewhlt.
 bsr sendctrl
 move.b #$55, d2 ;$55 wird in Register S0' (eigene I2C-Adresse) geschrieben.
 bsr senddata    ;Effektive I2C-Busadresse ist damit $AA (siehe Datenblatt)
                 ;Die Adresse ist unerheblich, weil der I2C-Prozessor nur als
                 ;Master eingestzt wird und von keinem anderen I2C-Device ange-
                 ;sprochen wird!
                 ;Aber der Schritt ist notwendig, damit der I2C-Prozessor
                 ;erkennt, ob es sich um einen INTEL- oder Motorola-Bus handelt!
 move.b #$A0, d1 ;Register S2 (Clock-Register) wird ausgewhlt
 bsr sendctrl
 move.b #$18, d2 ;$18 wird in Register S2 (Clockregister) geschrieben.
 bsr senddata    ;Es wird 8 MHz Systemtakt und 90 KHz I2C-SDL-Takt gesetzt.
 move.b #$C1, d1 ;Im Statusregister S1 wird das serielle Interface des I2C-
 bsr sendctrl    ;Busses aktiviert.
 move #10, d0    ;1 Sekunde warten
 moveq #!delay, d7
 trap #1
 rts             ;Ende Initialisierung

;* Master Transmitter Mode sendet Daten auf den I2C-Bus.
;* Parameter: sendbuff, #Bytes = d4, Slave-Adresse = d3
;* modebyte in d6:  00000abc mit:
;*      a: 0 = slave acknowledge        1 = no slave acknowledge
;*      b: 0 = stop condition           1 = no stop condition
;*      c: 0 = bus bysy check /         1 = no bus busy check /
;*             start condition              repeated start condition

mastertr:

;* Initialisierung
 lea sendbuff(a4), A0   ;Sendbuffer holen
 clr.l d5               ;Zhler fr gesendete Bytes initialisieren

;* bus busy check / start condition
 btst.b #0, d6
 bne.s repstart

busbusyt:
 bsr readctrl    ;Statusregister S1 auslesen
 btst.b #0, d1    ;Test auf Bit BB - bus busy
 beq.s busbusyt  ;wenn 0, dann warten und nochmal lesen

;* Start condition
 move.b d3, d2   ;I2C-Adresse des Peripheriebausteins bergeben (Slave-Adresse)
 bsr senddata    ;Ausgabe auf das S0-Register - Datenregister
 move.b #$C5, d1 ;Startcondition fr I2C-Bus wird ausgelst
 bsr sendctrl    ;SLAVE-Adresse wird gesendet
 bra.s transmit

;* Repeated Start condition
repstart:
 move.b d3, d2   ;I2C-Adresse des Peripheriebausteins bergeben (Slave-Adresse)
 bsr senddata    ;Ausgabe auf das S0-Register - Datenregister
 move.b #$45, d1
 bsr sendctrl

;*bertragung Daten aus Sendbuffer
transmit:
 bsr readctrl    ;Statusregister S1 auslesen
 btst.b #7, d1   ;Test auf PIN-Bit - bertragung beendet?
 bne.s transmit  ;Wenn 1, dann warten und nochmal Statusregister lesen

;* Acknowledege
 btst.b #2, d6
 bne.s sendbyte
 bsr readctrl    ;Slave acknowledged?
 btst #3, d1
 bne.s stopt     ;Wenn nicht, bertragung beenden

sendbyte:
 cmp.b d4,d5     ;Alle Bytes bertragen?
 beq.s stopt     ;Wenn ja, dann bertragung beenden
 addq #1, d5     ;Zhler fr bertragene Bytes um 1 inkrementieren
 move.b (A0)+, d2;nchstes Byte aus Sendbuffer holen
 bsr senddata    ;umd bertragen
 bra.s transmit  ;Schleife, solange noch nicht alle Bytes gesendet sind

;* Stop condition
 stopt:
 btst.b #1, d6
 bne.s endet
 move.b #$C3, d1 ;Stopcondition ber Controlregister auslsen
 bsr sendctrl

;* Ende Master Transmitter Mode
endet:
 move #20-1, d3         ;Warten bis zum naechsten Aufruf, sont zu schnell!
 mtrwait:
  nop
 dbra d3, mtrwait
rts

;* Routine Master Receiver Mode empfngt Daten vom I2C-Bus
;* Parameter: readbuff, #Bytes = d4, Slave-Adresse = d3.
;* modebyte in d6:  00000abc mit:
;*      a: 0 = slave acknowledge        1 = no slave acknowledge
;*      b: 0 = stop condition           1 = no stop condition
;*      c: 0 = bus bysy check /         1 = no bus busy check /
;*             start condition              repeated start condition

masterec:

;* Initialisierung

 lea readbuff(a4), A0   ;Readbuffer holen
 clr.l d5       ;Zhler fr empfangene Bytes initialisieren
 addq #1, d4    ;Zhler fr zu empfangende Bytes wird um 1 inkrementiert, weil
                ;das erste gelesene Byte die Slave-Adresse ist und keine Daten

;* bus busy check / start condition
 btst.b #0, d6
 bne.s repstarr

busbusyr:
 bsr readctrl    ;Statusregister S1 auslesen
 btst.b #0, d1    ;Test auf Bit BB - bus busy
 beq.s busbusyr  ;wenn 0, dann warten und nochmal lesen

;* Start condition
 move.b d3, d2   ;I2C-Adresse des Peripheriebausteins bergeben (Slave-Adresse)
 bsr senddata    ;Ausgabe auf das S0-Register - Datenregister
 move.b #$C5, d1 ;Startcondition fr I2C-Bus wird ausgelst
 bsr sendctrl    ;SLAVE-Adresse wird gesendet
 bra.s receive

;* Repeated Start condition
repstarr:
 move.b d3, d2   ;I2C-Adresse des Peripheriebausteins bergeben (Slave-Adresse)
 bsr senddata    ;Ausgabe auf das S0-Register - Datenregister
 move.b #$45, d1
 bsr sendctrl

;* Empfang Daten im Readbuffer
receive:
 bsr readctrl    ;Statusregister S1 auslesen
 btst.b #7, d1   ;Test auf PIN-Bit - bertragung beendet?
 bne.s receive  ;Wenn 1, dann warten und nochmal Statusregister lesen

;* Acknowledege
 btst.b #2, d6
 bne.s recbyte
 bsr readctrl    ;Slave acknowledged?
 btst #3, d1
 bne.s stopr     ;Wenn nicht, bertragung beenden

recbyte:
 cmp.b d4, d5    ;Alle Bytes empfangen, bis auf letztes?
 beq.s lastbyte  ;Wenn ja, dann letztes Byte behandeln
 addq #1, d5     ;Zhler fr bertragene Bytes um 1 inkrementieren
 bsr readdata    ;nchstes Byte emfangen
 cmp.b #1, d5    ;Als 1. Byte kommt die Slave-Adresse und wird nicht im
 beq.s receive   ;Readbuffer gespeichert
 move.b d2,(A0)+ ;nchstes Byte in Readbuffer schreiben
 bra.s receive   ; Schleife, bis vorletztes Byte empfangen wurde

lastbyte:
 move.b #$40, d1 ;negatives Acknowledgement wird in Controlregister gesetzt
 bsr sendctrl
 bsr readdata    ;Dies ist ein Dummyread der Slave-Adresse und wird nicht im
                 ;Readbuffer gespeichert
finishlb:
 bsr readctrl    ;Statusregister S1 auslesen
 btst #7, d1     ;Test auf PIN-Bit - bertragung beendet?
 bne.s finishlb  ;Wenn 1, dann warten und Statusregister erneut auslesen

;* Stop condition
 stopr:
 btst.b #1, d6
 bne.s ender
 move.b #$C3, d1 ;Stopcondition ber Controlregister auslsen
 bsr sendctrl
 bsr readdata    ;letztes zu empfangendes Byte holen
 move.b d2, (A0) ;und im Readbuffer speichern

;* Ende Master Receiver Mode
ender:
 move #20-1, d3         ;Warten bis zum naechsten Aufruf, sonst zu schnell!
 mrewait:
  nop
 dbra d3, mrewait
rts

;* Routinen zum Schreiben und Lesen des Kameramoduls
;* Das Modul besitzt den SCCB-Bus, der I2C-kompatible ist.

;* Schreiben:

kamerawr:
 move.b #IMAGEW, d3     ;I2C-Adresse zum Schreiben des Kameramoduls
 move d5, d4            ;Anzahl zu schreibende Bytes
 move.b #%00000000, d6  ;Bus busy check, Start cond., Acknowledge, Stop cond.
 bsr mastertr           ;und schreiben
 rts

;* Lesen

kamerard:
;* Erst Subadresse schreiben ...
 move.b #IMAGEW, d3     ;I2C-Adresse zum  Schreiben des Kameramoduls
                        ;Dummywrite mit Subadresse des device control registers
 move #1, d4            ;Anzahl zu sendender Bytes - hier 1 Byte, Subadresse
 move.b #%00000100, d6  ;Bus busy check/Start cond., no Stop cond., Acknowledge
 bsr mastertr

;* dann Lesen
 move.b #IMAGER, d3      ;I2C-Adresse zum Lesen des Kameramoduls
                         ;Jetzt Auslesen des Werts mit Leseroutine auf I2C-Bus
 move d5, d4             ;Anzahl zu lesender Bytes
 move.b #%00000101, d6   ;no Bus busy check/Rep. Start cond., Stop cond., Ackn
 bsr masterec            ;und Lesen - der Readbuffer wird durch die Routine
                         ;masterec gefllt!
 rts



; *** Schicht 2 - Bildschirm#maske


;* Grafik-Subroutinen

balkenh4:
        moveq #4-1, d4
        bra.s blkh01
balkenh2:
        moveq #2-1, d4
blkh01:
        asr #1, d2
        move #511, d3
blkh02:
        moveq #!moveto, d7
        trap #1
        exg d1, d3
        moveq #!drawto, d7
        trap #1
        exg d3, d1
        subq #1, d2
        dbra d4, blkh02
         rts

balkens:
        moveq #3-1, d4
        asr #1, d2
        move #0, d3
blk01:
        moveq #!moveto, d7
        trap #1
        exg d2, d3
        moveq #!drawto, d7
        trap #1
        exg d3, d2
        addq #1, d1
        dbra d4, blk01
         rts

;* Figuren

fstop:          ; STOP-Zeichen
        move.b #4, d0
        move.w #75, d1
        move.w #404, d2
        move.w #20, d3
        moveq #!grafik, d7
        trap #1
        rts

fplay:          ; PLAY-Zeichen
        move #75, d5
        move #202, d6
        dreieck:
        move d5, d1
        move d6, d2
        moveq #!moveto, d7
        trap #1
        move #94, d1
        move #206, d2
        moveq #!drawto, d7
        trap #1
        addq #1, d6
        cmp #210, d6
        ble.s dreieck
        rts

;* Lschroutinen

eraimg:         ; Lscht Bildanzeige COL256

movea.l #$90000, A0
move #256-1, d6         ;Alle 240 Zeilen von QVGA (obwohl die COL256 256 kann)
frame:                  ;Bild
 move #256-1, d5        ;Punkte / Zeile von QVGA mit 320
 line:                  ;Zeile
  move.b $FF, (A0)+     ;Bildpunkt
 dbra d5, line          ;1 Zeile verarbeitet
dbra d6, frame          ;Alle Zeilen - das ganze Bild - verarbeitet
rts

erastat:        ;Lscht Status

 moveq #!erapen, d7      ;GDP auf Lschen
 trap #1
 move #22, d1
 move #396, d2
 move #122, d3
 move #36, d4
 moveq #6, d0
 moveq #!grafik, d7
 trap #1
 moveq #!setpen, d7
 trap #1
rts

eramod:         ;Lscht Modus

 moveq #!erapen, d7      ;GDP auf Lschen
 trap #1
 move #147, d1
 move #396, d2
 move #122, d3
 move #36, d4
 moveq #6, d0
 moveq #!grafik, d7
 trap #1
 moveq #!setpen, d7
 trap #1
rts

eradir:         ;Lscht Directory

 moveq #!erapen, d7      ;GDP auf Lschen
 trap #1
 move #20, d1
 move #206, d2
 move #464, d3
 move #164, d4
 moveq #6, d0
 moveq #!grafik, d7
 trap #1
 moveq #!setpen, d7
 trap #1
rts

;* Bildschirmmaske

maske:

;* Hauptrahmen / Titel

;oben
 move #0, d1
 move #511, d2
 bsr balkenh4
;unten
 move #0, d1
 move #7, d2
 bsr balkenh4
;links
 move #0, d1
 move #507, d2
 bsr balkens
;rechts
 move #509, d1
 move #507, d2
bsr balkens
;Titel
 move #0, d1
 move #470, d2
 bsr balkenh2
 move.b #$21, d0
 move.w #23, d1
 move.w #239, d2
 lea titel(PC), a0
 move #!write, d7
 trap #1

 ;* Rahmen Inhaltsverzeichnis

  clr d1
  move #382, d2
  bsr balkenh2
  move.b #%00001000, GDP2.w             ; Vertikalschrift "directory"
  move.b #$12, GDP3.w
  move #500, d1
  move #110, d2
  lea lbldir, A0
  move #9-1, d3
  prtschl:
  moveq #!moveto, d7
  trap #1
  move.b (A0)+, d0
  moveq #!cmd, d7
  trap #1
  addq #7, d2
  dbra d3, prtschl

;* Rahmen Datum / Uhrzeit

 move #293 ,d1
 move #395, d2
 move #200, d3
 move #40, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 move.b #$11, d0
 move.w #293, d1
 move.w #219, d2
 lea datuhr(PC), a0
 moveq #!write, d7
 trap #1

;* Rahmen Status und Bildmodus

 move #145 ,d1
 move #395, d2
 move #125, d3
 move #40, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 move #20, d1
 move #395, d2
 move #125, d3
 move #40, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 bsr fstop
 move.b #$11, d0
 move.w #20, d1
 move.w #219, d2
 lea lblstat(PC), a0
 moveq #!write, d7
 trap #1
 move.b #$22, d0
 move.w #180, d1
 move.w #198, d2
 lea modvid(PC), a0
 moveq #!write, d7
 trap #1

;* Rahmen Bildeinstellung

 move #20,d1
 move #19, d2
 move #475, d3
 move #44, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 move.b #$11, d0
 move #40, d1
 move #16, d2
 lea hells(PC), a0
 moveq #!write, d7
 trap #1
 move #268, d1
 lea konts(PC), a0
 moveq #!write, d7
 trap #1
 move #150,d1
 move #28,d2
 move #107, d3
 move #25, d4
 moveq #5 , d0
 moveq #!grafik, d7
 trap #1
 move #366, d1
 moveq #!grafik, d7
 trap #1

;* Rahmen Funktion

 clr d1
 move #130, d2
 bsr balkenh2
 move #20,d1
 move #77, d2
 move #475, d3
 move #24, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 move.b #$11, d0
 move #20, d1
 move #52, d2
 lea lblfunkt(PC), a0
 moveq #!write, d7
 trap #1
 move #40, d1
 move #40, d2
 lea funkt1(PC), a0
 moveq #!write, d7
 trap #1
 move #335, d1
 lea funkt2(PC), A0
 moveq #!write , d7
 trap #1

;* Rahmen JADOS

 move #20,d1
 move #140, d2
 move #475, d3
 move #44, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 move.b #$11, d0
 move #20, d1
 move #94, d2
 lea lbljados, a0
 moveq #!write, d7
 trap #1
 move #40, d1
 move #76, d2
 lea drives(PC), a0
 moveq #!write, d7
 trap #1
 move #149, d1
 lea filesavs(PC), a0
 moveq #!write, d7
 trap #1
 move #329, d1
 lea fileloas(pc), a0
 moveq #!write, d7
 trap #1
 move #84,d1
 move #150,d2
 move #26, d3
 move #25, d4
 moveq #5, d0
 moveq #!grafik, d7
 trap #1
 move #218, d1
 move #75, d3
 moveq #!grafik, d7
 trap #1
 move #398, d1
 moveq #!grafik, d7
 trap #1
rts


; *** Schicht 3 - Routinen fr Steuerung Kameramodul, Bildanzeige / -speicherung
;                 und Speicherung der Bildeinstellungen

;* Initialisierung des Kameramoduls

caminit:

;* Bildschirm COL256 loeschen
 bsr eraimg

;* I2C-Prozessor initialisieren
 bsr initi2c

;* Initiale Einstellung fuer FIFO: RE = 1 (Bit 4),RCK = 0 (Bit 3),
;* WRST = 1 (Bit 2),WE = 0 (Bit 1), RRST = 1 (Bit 0)
  move.b #%00010101, FIFOwr
  move.b #%00011101, FIFOwr     ;RCK fr Uebernahme RRST
  move.b #%00010101, FIFOwr     ;RCK

;* Softwarereset fr Kameramodul
 lea sendbuff(A4), A0
 move.b #COM07, (A0)+   ;I2C-Subadresse
 move.b #$80, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 nop                    ;1 ms warten, bis RESET erfolgt

;* Product ID MSB fr Funktionscheck I2C abfragen
 lea sendbuff(A4), A0
 move.b #PID, (A0)      ;I2C-Subadresse
 move #1, d5            ;Anzahl Bytes
 bsr kamerard           ;von Kameramodul lesen
 lea readbuff(A4), A0
 move.b (A0), d0
 cmp.b #$76, d0         ;Wert muss $76 sein!
 beq.s initend          ;Wenn korrekter Wert, dann Ende der Initialisierung
 clr.b endeflag(A4)     ;Flag fr Programmabbruch
 lea err01, A1
 bsr status
initend:
rts

;* Basiseinstellung des Kameramoduls
;* Es werden nur bestimmte Einstellungen gegenber den Default-Werten gendert.
;* Die Einstellungen sind aus dem software application note der Kamera entnom-
;* men. Sie sind z.T. auch nicht logisch ...

camsetup:

;* Parameterbuffer laden

lea parbuff (A4), A1

move.b #BLUE, (A1)+
move.b #pblue, (A1)+
move.b #RED, (A1)+
move.b #pred, (A1)+
move.b #VREF, (A1)+
move.b #pvref, (A1)+
move.b #COM03, (A1)+
move.b #pcom03, (A1)+
move.b #COM05, (A1)+
move.b #pcom05, (A1)+
move.b #COM06, (A1)+
move.b #pcom06, (A1)+
move.b #AECH, (A1)+
move.b #paech, (A1)+
move.b #COM07, (A1)+
move.b #pcom07v, (A1)+
move.b #COM08, (A1)+
move.b #pcom08, (A1)+
move.b #COM09, (A1)+
move.b #pcom09, (A1)+
move.b #RSVD01, (A1)+
move.b #prsvd01, (A1)+
move.b #HSTAR, (A1)+
move.b #phstar, (A1)+
move.b #HSTOP, (A1)+
move.b #phstop, (A1)+
move.b #VSTAR, (A1)+
move.b #pvstar, (A1)+
move.b #VSTOP, (A1)+
move.b #pvstop, (A1)+
move.b #MVFP, (A1)+
move.b #pmvfp, (A1)+
move.b #ADCCTR1, (A1)+
move.b #padcctr1, (A1)+
move.b #ADCCTR2, (A1)+
move.b #padcctr2, (A1)+
move.b #ADCCTR3, (A1)+
move.b #padcctr3, (A1)+
move.b #AEW, (A1)+
move.b #paew, (A1)+
move.b #AEB, (A1)+
move.b #paeb, (A1)+
move.b #VPT, (A1)+
move.b #pvpt, (A1)+
move.b #RSVD02, (A1)+
move.b #prsvd02, (A1)+
move.b #HREF, (A1)+
move.b #phref, (A1)+
move.b #CHLF, (A1)+
move.b #pchlf, (A1)+
move.b #ARBLM, (A1)+
move.b #parblm, (A1)+
move.b #RSVD03, (A1)+
move.b #prsvd03, (A1)+
move.b #ADC, (A1)+
move.b #padc, (A1)+
move.b #ACOM, (A1)+
move.b #pacom, (A1)+
move.b #OFON, (A1)+
move.b #pofon, (A1)+
move.b #TSLB, (A1)+
move.b #ptslb, (A1)+
move.b #COM11, (A1)+
move.b #pcom11, (A1)+
move.b #COM12, (A1)+
move.b #pcom12, (A1)+
move.b #COM13, (A1)+
move.b #pcom13, (A1)+
move.b #COM14, (A1)+
move.b #pcom14, (A1)+
move.b #COM15, (A1)+
move.b #pcom15, (A1)+
move.b #COM16, (A1)+
move.b #pcom16, (A1)+
move.b #AWBC3, (A1)+
move.b #pawbc3, (A1)+
move.b #AWBC4, (A1)+
move.b #pawbc4, (A1)+
move.b #AWBC5, (A1)+
move.b #pawbc5, (A1)+
move.b #AWBC6, (A1)+
move.b #pawbc6, (A1)+
move.b #DMPOS, (A1)+
move.b #pdmpos, (A1)+
move.b #RSVD04, (A1)+
move.b #prsvd04, (A1)+
move.b #MTX1, (A1)+
move.b #pmtx1, (A1)+
move.b #MTX2, (A1)+
move.b #pmtx2, (A1)+
move.b #MTX3, (A1)+
move.b #pmtx3, (A1)+
move.b #MTX4, (A1)+
move.b #pmtx4, (A1)+
move.b #MTX5, (A1)+
move.b #pmtx5, (A1)+
move.b #MTX6, (A1)+
move.b #pmtx6, (A1)+
move.b #MTXS, (A1)+
move.b #pmtxs, (A1)+
move.b #AWBC7, (A1)+
move.b #pawbc7, (A1)+
move.b #AWBC8, (A1)+
move.b #pawbc8, (A1)+
move.b #AWBC9, (A1)+
move.b #pawbc9, (A1)+
move.b #AWBC10, (A1)+
move.b #pawbc10, (A1)+
move.b #AWBC11, (A1)+
move.b #pawbc11, (A1)+
move.b #AWBC12, (A1)+
move.b #pawbc12, (A1)+
move.b #GFIX, (A1)+
move.b #pgfix, (A1)+
move.b #GGAIN, (A1)+
move.b #pggain, (A1)+
move.b #AWBCTR3, (A1)+
move.b #pawbctr3, (A1)+
move.b #AWBCTR2, (A1)+
move.b #pawbctr2, (A1)+
move.b #AWBCTR1, (A1)+
move.b #pawbctr1, (A1)+
move.b #AWBCTR0, (A1)+
move.b #pawbctr0, (A1)+
move.b #SCALX, (A1)+
move.b #pscalx, (A1)+
move.b #SCALY, (A1)+
move.b #pscalyv, (A1)+
move.b #SCALDCW, (A1)+
move.b #pscaldcw, (A1)+
move.b #SCALCLK, (A1)+
move.b #pscalclk, (A1)+
move.b #REG74, (A1)+
move.b #preg74, (A1)+
move.b #REG75, (A1)+
move.b #preg75, (A1)+
move.b #REG76, (A1)+
move.b #preg76, (A1)+
move.b #RSVD05, (A1)+
move.b #prsvd05, (A1)+
move.b #RGB444, (A1)+
move.b #prgb444, (A1)+
move.b #RSVD06, (A1)+
move.b #prsvd06, (A1)+
move.b #RSVD07, (A1)+
move.b #prsvd07, (A1)+
move.b #RSVD08, (A1)+
move.b #prsvd08, (A1)+
move.b #RSVD09, (A1)+
move.b #prsvd09, (A1)+
move.b #RSVD10, (A1)+
move.b #prsvd10, (A1)+
move.b #DMLNL, (A1)+
move.b #pdmlnl, (A1)+
move.b #RSVD11, (A1)+
move.b #prsvd11, (A1)+
move.b #RSVD12, (A1)+
move.b #prsvd12, (A1)+
move.b #RSVD13, (A1)+
move.b #prsvd13, (A1)+
move.b #RSVD14, (A1)+
move.b #prsvd14, (A1)+
move.b #RSVD15, (A1)+
move.b #prsvd15, (A1)+
move.b #RSVD16, (A1)+
move.b #prsvd16, (A1)+
move.b #RSVD17, (A1)+
move.b #prsvd17, (A1)+
move.b #BD50ST, (A1)+
move.b #pbd50st, (A1)+
move.b #BD60ST, (A1)+
move.b #pbd60st, (A1)+
move.b #HAECC1, (A1)+
move.b #phaecc1, (A1)+
move.b #HAECC2, (A1)+
move.b #phaecc2, (A1)+
move.b #RSVD18, (A1)+
move.b #prsvd18, (A1)+
move.b #CLKDEL, (A1)+
move.b #pclkdel, (A1)+
move.b #NTCTRL, (A1)+
move.b #pntctrl, (A1)+
move.b #BD50MAX, (A1)+
move.b #pbd50max, (A1)+
move.b #HAECC3, (A1)+
move.b #phaecc3, (A1)+
move.b #HAECC4, (A1)+
move.b #phaecc4, (A1)+
move.b #HAECC5, (A1)+
move.b #phaecc5, (A1)+
move.b #HAECC6, (A1)+
move.b #phaecc6, (A1)+
move.b #HAECC7, (A1)+
move.b #phaecc7, (A1)+
move.b #BD60MAX, (A1)+
move.b #pbd60max, (A1)+
move.b #RSVD19, (A1)+
move.b #prsvd19, (A1)+
move.b #ABLC1, (A1)+
move.b #pablc1, (A1)+
move.b #RSVD20, (A1)+
move.b #prsvd20, (A1)+
move.b #THLST, (A1)+
move.b #pthlst, (A1)+
move.b #RSVD21, (A1)+
move.b #prsvd21, (A1)+
move.b #SATCTR, (A1)+
move.b #psatctr, (A1)+
move.b #DBLV, (A1)+
move.b #pdblv, (A1)+
move.b #CLKRC, (A1)+
move.b #pclkrc, (A1)+
move.b #'*', (A1)       ;Pufferabschlus

lea parbuff(A4), A1
move #2, d5             ;Anzahl zu bertragender Bytes per I2C
setup:
 lea sendbuff(A4), A0   ;Sendbuffer I2C
 move.b (A1)+, (A0)+    ;I2C-Subadresse / Kameraregister
 move.b (A1)+, (A0)     ;Wert
 bsr kamerawr           ;Auf Kameramodul schreiben
 cmp.b #'*', (A1)       ;Parameterpuffer zuende?
 bne.s setup
rts

;* Einstellung von Video- oder Testbildwiedergabe aus dem Programm

setvid:
 lea sendbuff(A4), A0   ;Farbe einstellen
 move.b #COM07, (A0)+   ;I2C-Subadresse
 move.b #pcom07v, (A0)  ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

 lea sendbuff(A4), A0   ;Video einstellen
 move.b #SCALY, (A0)+   ;I2C-Subadresse
 move.b #pscalyv, (A0)  ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

 ;* Takt - zum Schluss setzen, damit alle Einstellungen wirksam werden:

 lea sendbuff(A4), A0   ;externer Takt 12 MHz X 4
 move.b #DBLV, (A0)+    ;I2C-Subadresse
 move.b #pdblv, (A0)    ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

 lea sendbuff(A4), A0   ;PLL-Takt / 2 = interner Takt 24 MHz
 move.b #CLKRC, (A0)+   ;I2C-Subadresse
 move.b #pclkrc, (A0)   ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
rts

settst:
 lea sendbuff(A4), A0   ;Testbild einstellen
 move.b #COM07, (A0)+   ;I2C-Subadresse
 move.b #pcom07t, (A0)  ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

 lea sendbuff(A4), A0   ;Testbild einstellen
 move.b #SCALY, (A0)+   ;I2C-Subadresse
 move.b #pscalyt, (A0)  ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

 ;* Takt - zum Schluss setzen, damit alle Einstellungen wirksam werden:

 lea sendbuff(A4), A0   ;externer Takt 12 MHz X 4
 move.b #DBLV, (A0)+    ;I2C-Subadresse
 move.b #pdblv, (A0)    ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

 lea sendbuff(A4), A0   ;PLL-Takt / 2 = interner Takt 24 MHz
 move.b #CLKRC, (A0)+   ;I2C-Subadresse
 move.b #pclkrc, (A0)   ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
rts

;* Routinen zum Einstellen von Helligkeit und Kontrast

setstell:       ;Setzt aktive Anzeige fr Helligkeit und Kontrast
 cmp.b #$00, stellart(A4)       ;Helligkeit
 bne.s stellk
 move #150, d1
 move #38, d2
 move #107, d3
 moveq #5, d4
 moveq #6, d0
 moveq #!grafik, d7             ;Markierung Helligkeit
 trap #1
 moveq #!erapen, d7             ;Entmarkierung Kontrast
 trap #1
 move #367, d1
 move #38, d2
 move #105, d3
 moveq #5, d4
 moveq #6, d0
 moveq #!grafik, d7
 trap #1
 moveq #!setpen, d7
 trap #1
 bra.s stellend

stellk:
 move #366, d1
 move #38, d2
 move #107, d3
 moveq #5, d4
 moveq #6, d0
 moveq #!grafik, d7             ;Markierung Kontrast
 trap #1
 moveq #!erapen, d7             ;Entmarkierung Helligkeit
 trap #1
 move #151, d1
 move #38, d2
 move #105, d3
 moveq #5, d4
 moveq #6, d0
 moveq #!grafik, d7
 trap #1
 moveq #!setpen, d7
 trap #1
stellend:
rts

stepb:          ; Setzt aktuellen Grafikzeiger fr Helligkeit und Kontrast bei
                ; bei Verminderung des Wertes
move.b stellart(A4), d1
cmp.b #$00, d1          ;Helligkeit
bne.s stbkont
clr d1
clr d3
move.b letzteh(A4), d1  ;alter Wert
move.b d1, d3
subq #1, d3             ;neuer Wert
cmp.b #0, d3            ;Minimalwert?
ble stepbend            ;dann nichts tun
move.b d3, letzteh(A4)  ;Wert speichern
bsr hellg
bra stepbend
stbkont:
clr d1
clr d3
move.b letztek(A4), d1  ;alter Wert
move.b d1, d3
subq #1, d3             ;neuer Wert
cmp.b #0, d3            ;Minimalwert?
ble.s stepbend          ;dann nichts tun
move.b d3, letztek(A4)  ;Wert speichern
bsr kontg
stepbend:
rts

stepf:          ; Setzt aktuellen Grafikzeiger fr Helligkeit und Kontrast
                ; Erhhung des Wertes
move.b stellart(A4), d1
cmp.b #$00, d1          ;Helligkeit
bne.s stfkont
clr d1
clr d3
move.b letzteh(A4), d1  ;alter Wert
move.b d1, d3
addq #1, d3             ;neuer Wert
cmp.b #5, d3            ;Maximalwert?
bgt stepfend            ;dann nichts tun
move.b d3, letzteh(A4)  ;Wert speichern
bsr hellg
bra stepfend
stfkont:
clr d1
clr d3
move.b letztek(A4), d1  ;alter Wert
move.b d1, d3
addq #1, d3             ;neuer Wert
cmp.b #5, d3            ;Maximalwert?
bgt.s stepfend          ;dann nichts tun
move.b d3, letztek(A4)  ;Wert speichern
bsr kontg
stepfend:
rts

hellg:           ;Helligkeit - Grafik

;* Alten Helligkeitswert lschen - in d1

 moveq #!erapen, d7     ;GDP auf Lschen
 trap #1
 subq #1, d1
 mulu #25, d1
 add #151, d1
 move #16, d2
 moveq #!moveto, d7
 trap #1
 move.b #$11, GDP3.w
 lea fig1(pc), a0
 moveq #!progzge, d7
 trap #1

;* Neuen Helligkeitswert anzeigen - in d3

hellinit:
 clr d1
 moveq #!setpen, d7     ;GDP auf Schreiben
 trap #1
 move.b d3, d1
 subq #1, d1
 mulu #25, d1
 add #151, d1
 move #16, d2
 moveq #!moveto, d7
 trap #1
 move.b #$11, GDP3.w
 lea fig1(pc), a0
 moveq #!progzge, d7
 trap #1
 bsr camp               ;Einstellung Helligkeit
 rts

kontg:                  ;Kontrast - Grafik

;* Alten Kontrastwert lschen - in d1

moveq #!erapen, d7      ;GDP auf Lschen
 trap #1
 subq #1, d1
 mulu #25, d1
 add #367, d1
 move #16, d2
 moveq #!moveto, d7
 trap #1
 move.b #$11, GDP3.w
 lea fig1(pc), a0
 moveq #!progzge, d7
 trap #1

;* Neuen Kontrastwert anzeigen - in d3

kontinit:
 clr d1
 moveq #!setpen, d7     ;GDP auf Schreiben
 trap #1
 move.b d3, d1
 subq #1, d1
 mulu #25, d1
 add #367, d1
 move #16, d2
 moveq #!moveto, d7
 trap #1
 move.b #$11, GDP3.w
 lea fig1(pc), a0
 moveq #!progzge, d7
 trap #1
 bsr camp               ;Einstellung Kontrast
 rts

;* Einstellwerte an das Kameramodul bergeben

camp:

 move.b stellart(A4), d0        ;"Helligkeit" oder "Kontrast"
 cmp.b #0, d0
 bne kontp

;* Helligkeit

 cmp.b #1, d3
 bne.s hell2
 lea sendbuff(A4), A0   ;Helligkeit -2
 move.b #BRIGHT, (A0)+  ;I2C-Subadresse
 move.b #$B0, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 hell2:
 cmp.b #2, d3
 bne.s hell3
 lea sendbuff(A4), A0   ;Helligkeit -1
 move.b #BRIGHT, (A0)+  ;I2C-Subadresse
 move.b #$98, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 hell3:
 cmp.b #3, d3
 bne.s hell4
 lea sendbuff(A4), A0   ;Helligkeit -0
 move.b #BRIGHT, (A0)+  ;I2C-Subadresse
 move.b #$00, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 hell4:
 cmp.b #4, d3
 bne.s hell5
 lea sendbuff(A4), A0   ;Helligkeit +1
 move.b #BRIGHT, (A0)+  ;I2C-Subadresse
 move.b #$18, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 hell5:
 cmp.b #5, d3
 lea sendbuff(A4), A0   ;Helligkeit +2
 move.b #BRIGHT, (A0)+  ;I2C-Subadresse
 move.b #$30, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend

;* Kontrast

kontp:
 cmp.b #1, d3
 bne.s kont2
 lea sendbuff(A4), A0   ;Helligkeit -2
 move.b #CONTRA, (A0)+  ;I2C-Subadresse
 move.b #$30, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 kont2:
 cmp.b #2, d3
 bne.s kont3
 lea sendbuff(A4), A0   ;Helligkeit -1
 move.b #CONTRA, (A0)+  ;I2C-Subadresse
 move.b #$38, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 kont3:
 cmp.b #3, d3
 bne.s kont4
 lea sendbuff(A4), A0   ;Helligkeit -0
 move.b #CONTRA, (A0)+  ;I2C-Subadresse
 move.b #$40, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 kont4:
 cmp.b #4, d3
 bne.s kont5
 lea sendbuff(A4), A0   ;Helligkeit +1
 move.b #CONTRA, (A0)+  ;I2C-Subadresse
 move.b #$50, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
 bra campend
 kont5:
 lea sendbuff(A4), A0   ;Helligkeit +2
 move.b #CONTRA, (A0)+  ;I2C-Subadresse
 move.b #$60, (A0)      ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

campend:

 ;* Takt - zum Schluss setzen, damit alle Einstellungen wirksam werden:

 lea sendbuff(A4), A0   ;externer Takt 12 MHz X 4
 move.b #DBLV, (A0)+    ;I2C-Subadresse
 move.b #pdblv, (A0)    ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben

 lea sendbuff(A4), A0   ;PLL-Takt / 2 = interner Takt 24 MHz
 move.b #CLKRC, (A0)+   ;I2C-Subadresse
 move.b #pclkrc, (A0)   ;Wert
 move #2, d5            ;Anzahl Bytes
 bsr kamerawr           ;Auf Kameramodul schreiben
rts



 rts


;* Bildaufnahme und Speichern im FIFO-RAM des Kameramoduls

camcapt:

;* Ausgabeport Status
;* WriteReset aktivieren (Bit 2 = 0)
move.b #%00010001, FIFOwr       ;WRST
nop
move.b #%00010101, FIFOwr       ;Betrieb

;* Start FRAME

;* Eingabeport Status
;* VertSync = Bit 0
;* Bit 7-1 nicht relevant

;* Wechselt Wert von Bit 0 von "1" auf "0", dann beginnt ein FRAME
 vsyncba:
  move.b VERTsync, d0    ;Test auf "1"
  btst.b #0, d0
  beq.s vsyncba
 vsyncbb:
  move.b VERTsync, d0    ;Test auf "0"
  btst.b #0, d0
  bne.s vsyncbb

;* WriteEnable FIFO-RAM
;* Auf dem Kameramodul wird WriteEnable mit HORIZONTAL-REF UND-verknpft!
;* Mit jedem Puls HORIZONTAL-REF werden Bilddaten in das FIFO-RAM geschrieben.

;WriteEnable FIFO
;* Ausgabeport Status
 move.b #%00010111, FIFOwr      ;WE=1

;* End FRAME

;* Eingabeport Status
;* VertSync = Bit 0
;* Bit 7-1 nicht relevant

;* Erreicht der Wert von Bit 0 wieder "1", dann endet ein FRAME
vsyncbe:
 move.b VERTsync, d0
 btst.b #0, d0
 beq.s vsyncbe

;* WriteDisable FIFO
;* Ausgabeport Status
 move.b #%00010101, FIFOwr      ;WE=0

;* Das Bild steht nun im FIFO-RAM mit Format RGB444

 rts

;* Bild aus FIFO-RAM in NKC-Speicher bertragen
;* Es mssen 480 (Zeilen) X 256 (Punkte / Zeile) X 2 (Bytes / Punkt) bertragen
;* werden!
;* Zudem muss das Datenformat RGB4444 in das NKC-Format der COL256 gewandelt
;* werden - also von R6,R5,R4,R3,G6,G5,G4,G3,B6,B5,B4,B3,0,0,0,0
;*                in II,BB,GG,RR , wobei II die Intensitt aller Farben ist.

nkccapt:

;* Ausgabeport Status
;* ReadReset (Bit 0=0)
move.b #%00010100, FIFOwr
move.b #%00011100, FIFOwr       ;RCK
move.b #%00010101, FIFOwr       ;RCK
move.b #%00000101, FIFOwr

;* Bild-Zieladresse fr COL256
movea.l #$90000, A0

;* Daten aus dem FIFO-RAM umrechnen und in den NKC-Speicher bertragen

 clr.l d0

 move #240-1, d6        ;Alle 240 Zeilen von QVGA (obwohl die COL256 256 kann)
 bildc:
  move #256-1, d5       ;Punkte / Zeile fr COL256 (obwohl QVGA 320 liefert)
  zeilec:               ;Erfasste Zeile
   move.b #%00001101, FIFOwr    ;RCK=1
   move.b FIFOrd, d0    ;EVEN-Byte 00,00,00,00,R6,R5,R4,R3
   lsl #8, d0
   move.b #%00000101, FIFOwr    ;RCK=0
nop
   move.b #%00001101, FIFOwr    ;RCK=1
   move.b FIFOrd, d0    ;ODD-Byte  G6,G5,G4,G3,B6,B5,B4,B3
   move.b #%00000101, FIFOwr    ;RCK=0

;* D0 enthlt nun 1 Punkt mit EVEN- und ODD-Byte als Wort in RGB444

   ;* Saparieren der Farbwerte:
   clr d1
   clr d2
   clr d3

   move.b d0, d1
   and.b #%00001111, d1         ;D1.b enthlt nur Blauwerte
   move.w d0, d2
   lsr #4, d2
   and.b #%00001111, d2         ;D2.b enhlt nur Grnwerte
   move.w d0, d3
   lsr #8, d3
   and.b #%00001111, d3         ;D3.b enthlt nur Rotwerte

   ;* Umrechnen in NKC-Format, D4.b enthlt am Ende das Byte eines Punktes:

   ;Intensitt - UND-Verknuepfung der Farbwerte, Ergebnis in Bit 7, 6 von d4.
   ;Es werden nur diue beiden niederwertigen 2 der 4 Bits betrachtet.

   clr d4
   move.b d1, d4                ;Blau
   and.b  d2, d4                ;Grn
   and.b d3, d4                 ;Rot
   lsl #6, d4                   ;in D4 steht die Intensitt in Bit 7, 6

   ;Farbwerte - es werden nur die hchstwertigen 2 der 4 Bits betrachtet:

   lsl #2, d1
   and.b #%00110000, d1
   or.b d1, d4                  ;in d4 steht nun der Blauwert in Bit 5, 4
   and.b #%00001100, d2
   or.b d2, d4                  ;in d4 steht nun der Grnwert  in Bit 3, 2
   lsr #2, d3
   and.b #%00000011, d3
   or.b d3, d4                  ;in D4 steht der Rotwert in Bit 1, 0

   ;Bildpunkt als NKC-Byte IIBBGGRR in den NKC-Speicher schreiben:

   move.b d4, (A0)
   adda.l #1, A0

  dbra d5, zeilec               ;1 Zeile verarbeitet
 dbra d6, bildc                 ;Alle Zeilen - das ganze Bild - verarbeitet
 rts




; *** Schicht 4 - Hauptprogramm


cam:                    ;Label Hauptprogrammstart

;* Bildseiteneinstellung - versteckter Grafikaufbau

 move #!clr, d7
 trap #1                ;Alle Bildschirmseiten lschen
 clr.l d0
 clr.l d1
 moveq #!setflip, d7
 trap #1                ;Seitenumschaltung aus
 move.b #1, d0
 clr.l d1
 moveq #!newpage, d7    ;Schreibseite = 1, Leseseite = 0
 trap #1                ;Grafikaufbau im Hintergrund

;* Initialisierung

 bsr varinit            ;Variablen inititialisieren
 lea varstart(PC), A4   ;Variablenstart
 bsr maske              ;Aufbau Bildschirmmaske GDP64
 bsr colinit            ;Initialisierung COL256
 bsr caminit            ;Initialisierung I2C und Kameramodul
 bsr camsetup           ;Basiseinstellung Kameramodul fr GDP64K
 bsr setstell           ;Stellart wird mit varinit auf "Helligkeit" gesetzt
 move.b letzteh(A4), d3 ;Helligkeit wir auf Mitte gesetzt
 bsr hellinit
 move.b letztek(A4), d3 ;Kontrast wird auf Mitte gesetzt
 bsr kontinit
 bsr jpresent           ;Prueft, ob JADOS geladen ist
 bsr actdrive           ;Ermittelt aktuelles Laufwerk

;* Bildseiteneinstellung - Grafik anzeigen

move.b #1, d0
move.b #1, d1
moveq #!newpage, d7     ;Schreibseite = Leseseite = 1 - Grafik zeigen
trap #1

;* Hauptprogrammschleife:
;* Bedienung des Kameramoduls
;* JADOS Handling

eingabeo:

;* Stndige Abfragen

 bsr uhr                                ;Datum und Uhrzeit
 move.b endeflag(A4), d1                ;Programm beendet - regulr oder wenn
 beq fertig                             ;kein Kameramodul angeschlossen ist

 tast:
  moveq #!csts, d7                      ;Abfrage auf Tastatureingabe
  trap #1
  beq.s eingabeo
  bsr ciint                             ;Eingabe auswerten
  trap #1

  lcamcapt:
   cmp.b #'s', d0                       ;Bild erfassen
   bne.s lclr
   bsr erastat
   bsr fplay
   bsr camcapt
   bsr nkccapt
   bsr fstop
   bra eingabeo
  lclr:                                 ;Bildanzeige lschen
   cmp.b #'c', d0
   bne.s ldir
   bsr eraimg
   bra eingabeo
  ldir:
   cmp.b #'d', d0
   bne.s lsave
   bsr eradir
   bsr direc
   bra eingabeo
  lsave:
   cmp.b #'v', d0                       ;Speichern des Bilds als Datei
   bne.s lload
   bsr erastat
   lea sta01, A1
   bsr status
   bsr imgsave
   bsr erastat
   bsr fstop
   bra eingabeo
 lload:
  cmp.b #'l', d0
  bne.s lmodus
  bsr erastat
  lea sta02, A1
  bsr status
  bsr imgload
  bsr erastat
  bsr fstop
  bra eingabeo
 lmodus:
  cmp.b #'m', d0                ;"Video" oder "Testbild"
  bne.s lstepf
  move.b modus(A4), d1          ;aktuellen Modus holen
  cmp.b #$00, d1
  beq.s ltest                   ;"Video" gesetzt, "Testbild" wird gesetzt
  bsr eramod
  move.b #$00, modus(A4)        ;"Testbild" gestzt, "Video" wird gesetzt
  move.b #$22, d0
  move #180, d1
  move #198, d2
  lea modvid, A0
  moveq #!write, d7
  trap #1
  bsr setvid                    ;"Video wird gesetzt"
  bra eingabeo
  ltest:
   move.b #$01, modus(A4)
   bsr eramod
   move.b #$22, d0
   move #180, d1
   move #198, d2
   moveq #!write, d7
   trap #1
   lea modtst, A0               ;"Testbild" wird gesetzt
   moveq #!write, d7
   trap #1
   bsr settst
   bra eingabeo
  lstepf:                       ;Helligkeit, Kontrast erhoehen
   cmp.b #'+', d0
   bne.s lstepb
   bsr stepf
  lstepb:                       ;Helligkeit, Kontrast vermindern
   cmp.b #'-', d0
   bne.s lhell
   bsr stepb
  lhell:
   cmp.b #'b', d0               ;Stellart "Helligkeit" setzen
   bne.s lkont
   move.b #$00, stellart(A4)
   bsr setstell
  lkont:
   cmp.b #'o', d0               ;Stellart "Kontrast" setzen
   bne.s ldrive
   move.b #$01, stellart(A4)
   bsr setstell
  ldrive:
   cmp.b #'r', d0               ;Laufwerk setzen
   bne.s lende
   bsr setdrive                 ;LW setzen
   bsr actdrive                 ;Aktuelles LW anzeigen
  lende:
   cmp.b #$1B, d0               ;ESC-Taste - Programmende
   bne eingabeo
   clr.b endeflag(A4)
   bra sfertig
  fertig:
   moveq #30, d0                ;1 Sekunde warten
   moveq #!delay, d7
   trap #1
  sfertig:
   lea sta03, A1                ;Ausgabe Ende-Meldung
   bsr status
   moveq #20, d0                ;1 Sekunde warten
   moveq #!delay, d7
   trap #1
   moveq #!clr, d7              ;Bildschirm lschen
   trap #1
   rts



;*** Schicht 5 - Vereinbarungsbaustein - Variablen


varstart:

;* Datenpuffer fr I2C-Bus, Parameter fuer Camsetup und JADOS
;* stop-Flag fr Senden mit / ohne Stopcondition

sendbuff        equ 0                   ;  10 Byte
readbuff        equ sendbuff+10         ;  10 Byte
stop            equ readbuff+10         ;   1 Byte
parbuff         equ stop+1              ; 800 Byte

;* Variablen fr Bildeinstellungen Kameramodul
;* Helligkeit, Kontrast, Prameterbuffer
;* Stellart, Farbmodus

letzteh         equ parbuff+800         ;  1 Byte
letztek         equ letzteh+1           ;  1 Byte
stellart        equ letztek+1           ;  1 Byte
modus           equ stellart+1          ;  1 Byte
wertuhr         equ modus+1             ; 40 Byte

;* Flags / Einzelwerte

endeflag        equ wertuhr+40          ;  1 Byte
datmin          equ endeflag+1          ;  1 Byte
jados           equ datmin+1            ;  1 Byte
index           equ jados+1             ;  1 Byte

varanz          equ *-varstart

end.

